# -*- coding: utf-8 -*-
u"""
Internal module configuring ACE as a Django app.
"""
from __future__ import absolute_import, division, print_function

from django.apps import AppConfig


class EdxAceConfig(AppConfig):
    u"""
    Configuration for the edx_ace Django application.
    """

    name = u'edx_ace'
